#!/usr/local/bin/gawk -f
#
#	ship.gawk 		(c) GPL Michael Koehne kraehe@bakunin.north.de
#
#		shipdesign based on FF&S (c) GDW 
#		usage: just start it and type help
#                      Y'll need FF&S or Briliant lances to use it
#
# ----------------------------------------------------------------------------

BEGIN {
	tl=9
	dt=100
	jump=1
	jdisp=0
	graph=1
	jdisp=0
	ghour=30
	mvm=1
	lm=1
	ar=10
	mw=0
	cg=0
	ls="e"
	fpp=0
	add=0

	if (!system("tty >/dev/null")) { calc(); show() }
	}

# ----------------------------------------------------------------------------

$1 == "tl"  {	tl=$2				}
$1 == "dt"  {	dt=$2; jdisp=$2; gdisp=$2	}
$1 == "j"   {	jump=$2; if ($3) jdisp=$3	}
$1 == "g"   {	graph=$2; if ($3) gdisp=$3	}
$1 == "gh"  {	ghour=$2			}
$1 == "mvm" {	mvm=$2				}
$1 == "lm"  {	lm=$2				}
$1 == "ar"  {	ar=$2				}
$1 == "mw"  {	mw=$2				}
$1 == "cg"  {	cg=$2				}
$1 == "ls"  {	ls=$2				}
$1 == "fpp" {	fpp=$2				}
$1 == "add" {	
		add=++add
		add_type[add]=$3
		add_num [add]=$4
		add_mw  [add]=$5
		add_vol [add]=$6
		add_mass[add]=$7
		add_qm  [add]=$8
		if ($9 != ":" )
			add_sect[add]=$9
		add_name[add]=$10
		for (i=11; i<=NF; i++) add_name[add]=add_name[add] " " $i
		}

NF == 0 {	calc(); show()			}

# ----------------------------------------------------------------------------

$1 == "help" {
	printf "\nShipdesign\t\t(c) GPL Michael Koehne kraehe@bakunin.north.de\n"
	printf "\t\t\t    based on FF&S (c) GDW\n\n"
	printf "\tTech Level         tl=%d\t", tl
	printf "\tDisplacement       dt=%d\n", dt
	printf "\tJump performance    j=%d\t", jump
	printf "\tG performance       g=%d\n", graph
	printf "\tG hours            gh=%d\t", ghour
	printf "\tMaterial vol.mod  mvm=%d\n", mvm
	printf "\tLength modifier    lm=%d\t", lm
	printf "\tArmor value        ar=%d\n", ar
	printf "\tContra Graph       cg=%d\t", cg
	printf "\tFuel PP hours     fpp=%d\n", fpp
	printf "\tLife Support [beg] ls=%s\t", ls
	printf "\tadditional MW      mw=%d\n", mw
	printf "\nto change any value type : key<SPC>value<LF>\n"
	printf "to recalulate type : <LF>\n\n"
	}

# ----------------------------------------------------------------------------

function calc() {
#							Dimensions
	len=exp(log(6*dt*14/3.1416)/3)
	mv=len*len*3.1416/100
	len=len*lm
	mv=mv*mvm

	mass=0; load=dt; smw=mw

#							Hull

	if (ar < graph*10) ar=graph*10

	if (tl < 10) {
		hulldisp = ar*mv/6;     hullmass = hulldisp*8
		intedisp = graph*mv/6;  intemass = intedisp*8 }
	else
	if (tl < 12) {
		hulldisp = ar*mv/8;     hullmass = hulldisp*10
		intedisp = graph*mv/8;  intemass = intedisp*10 }
	else
	if (tl < 14) {
		hulldisp = ar*mv/14;    hullmass = hulldisp*15
		intedisp = graph*mv/14; intemass = intedisp*15 }
	else {	hulldisp = ar*mv/28;    hullmass = hulldisp*15
		intedisp = graph*mv/28; intemass = intedisp*15 }

	hulldisp /= 14;
	intedisp /= 14;

	mass += hullmass+intemass
	load -= hulldisp+intedisp

#							Jump drive

	jdrivedisp=jump ? jdisp*(0.01+jump/100): 0
	if (tl<14) jdrivemass=jdrivedisp*3
	if (tl==14) jdrivemass=jdrivedisp*2.5
	if (tl==15) jdrivemass=jdrivedisp*2
	jfuel=jdrivedisp*5

	mass += jdrivemass+jfuel
	load -= jdrivedisp+jfuel

#							Thusters

 	gwatt=graph*gdisp/2
 	thust=gwatt*0.1
 	gfuel=dt/(8*14)

	mass += thust+ghour*gfuel
	load -= thust/14+ghour*gfuel
	smw  += gwatt

#							Fuel purifcation

	if (fpp>0) {
	if (tl < 10) { fppmw = 0.009; fppvol=0.6;  }
	else
	if (tl < 11) { fppmw = 0.008; fppvol=0.55; }
	else
	if (tl < 12) { fppmw = 0.007; fppvol=0.45; }
	else
	if (tl < 13) { fppmw = 0.006; fppvol=0.4;  }
	else
	if (tl < 14) { fppmw = 0.005; fppvol=0.35; }
	else
	if (tl < 15) { fppmw = 0.005; fppvol=0.25; }
	else         { fppmw = 0.005; fppvol=0.2;  }

	fppmw  *= 6*(gfuel+jfuel)/fpp
	fppvol *= 6*(gfuel+jfuel)/fpp
	fppmass = fppvol*2

	mass += fppmass
	load -= fppvol/14
	smw  += fppmw
	}

#							Contra-Graph

	cgmw=0; cgvol=0; cgmass=0; 
	if (cg) {
		if (tl < 10) { cgmw=0.3; cgvol=0.5; cgmass=0.4; }
		else
		if (tl < 12) { cgmw=0.2; cgvol=0.3; cgmass=0.3; }
		else         { cgmw=0.1; cgvol=0.3; cgmass=0.2; }
		}

	cgmw   *= dt
	cgvol  *= dt
	cgmass *= dt

	mass += cgmass
	load -= cgvol/14
	smw  += cgmw

#							Controlls

	if (tl < 10) {  csmw=0.0005; cstype="Computer" }
	else
	if (tl < 13) {	csmw=0.001;  cstype="Dynamic" }
	else {		csmw=0.001;  cstype="Holographic" }

	csmw  *= dt
	csvol  = dt*0.014
	csmass = dt*0.0014

	mass += csmass
	load -= csvol/14
	smw  += csmw

#							Computers

	compmw   = (tl*0.05-0.2)*3
	compvol  = (tl < 13 ? 9+tl-13 : 22-tl)*3
	compmass = compvol*0.2

	mass += compmass
	load -= compvol/14
	smw  += compmw

#							Life support

	if (ls=="b") { lsmw=0.0001; lsvol=0.005; lsmass=0.005; lstype="Basic" } 
	if (ls=="e") { lsmw=0.0001; lsvol=0.008; lsmass=0.008; lstype="Extendet"} 
	if (ls=="g") { lsmw=0.0051; lsvol=0.015; lsmass=0.025; lstype="Ext+Grav" } 

	lsmw   *= load*14	# I know its tricky to use free load here
	lsvol  *= load*14
	lsmass *= load*14
	lsload  = load*14

	mass += lsmass
	load -= lsvol/14
	smw  += lsmw

#							Additionals

	for (i =1; i<= add; i++) {
		mass += add_mass[i]*add_num[i]
		smw  += add_mw[i]*add_num[i]
		load -= add_vol[i]*add_num[i]/14
		}

#							Power Plant

	if (tl < 13) { ppdisp=smw/2; ppmass=ppdisp*4; ppfuel=smw*0.15 }
	else
	if (tl < 15) { ppdisp=smw/3; ppmass=ppdisp*3; ppfuel=smw*0.1 }
	else	     { ppdisp=smw/6; ppmass=ppdisp*2; ppfuel=smw*0.1 }

	ppdisp /= 14
	ppfuel /= 14

	mass += ppmass+ppfuel
	load -= ppdisp+ppfuel
	}

function show() {
#							Printing

	printf "General Data:\n"

	printf "\tDisplacement : %5.5s dt     \tTech Level : %5.5s\n",
		dt,tl
	printf "\tLength       : %5.1f meters \tVolume     : %5.0d m3\n",
		len,dt*14
	printf "\tHull         : %5.0d m2     \tArmor      : %5.0d\n",
		mv*100,ar
	printf "\tMass         : %5.0d tons   \tLoad       : %5.0d dt\n",
		mass, load

	printf "\nEngineering Data:\n"

	printf "\tHull             : %d m3, %.1f dt, %d tons\n",
		hulldisp*14+intedisp*14, hulldisp+intedisp, hullmass+intemass
	printf "\tPower plant      : %d MW, %d m3, %.1f dt, %d tons\n",
		smw, (ppdisp+ppfuel)*14,ppdisp+ppfuel, ppmass+ppfuel
	printf "\tJump performance : %d (Drive %d m3, %.1f dt, Mass %d tons)\n",
		jump, jdrivedisp*14, jdrivedisp, jdrivemass
	printf "\tG-Rating         : %dG (Thusters %d MW, %d m3)\n",
		graph, gwatt, thust
	printf "\tG-Turns          : %d (%d using jump fuel) %.1f m3 of fuel each\n",
		ghour*2, 2*(ghour+jfuel/gfuel),7*gfuel
	if (cg)
	printf "\tContra Graph     : %d MW, %d m3, %.1f dt, %d tons\n",
		cgmw, cgvol, cgvol/14, cgmass
	if (fpp)
		printf "\tFuel scope       : %.1f m3, %d hours to refine",
			fppvol, fpp
	else
		printf "\tFuel             :"
	printf " %d m3, %.1f dt\n",
		14*ghour*gfuel+jfuel*14, ghour*gfuel+jfuel

	printf "\nElectronics:\n"
	printf "\tControll system  : %s linked %.2f MW, %.1f m3, %.2f tons\n",
		cstype, csmw, csvol, csmass
	printf "\tComputer         : 3*St %s %.2f MW, %.1f m3, %.2f tons\n",
		cstype, compmw, compvol, compmass
	for (i =1; i<= add; i++)
		if (add_type[i]=="elec") {
			printf "\t%-16.16s : ", add_sect[i]
			if (add_num[i]>1) printf "%d ", add_num[i]
			printf "%s ", add_name[i]
			if (add_mw[i]>=0.1) printf "%.1f MW ", add_mw[i]
			if (add_vol[i]>=0.1) printf "%.1f m3 ", add_vol[i]
			if (add_mass[i]>=0.1) printf "%.1f tons ", add_mass[i]
			printf "\n"
			}

	printf "\nArmament:\n"
	for (i =1; i<= add; i++)
		if (add_type[i]=="arma") {
			printf "\t%-16.16s : ", add_sect[i]
			if (add_num[i]>1) printf "%d ", add_num[i]
			printf "%s ", add_name[i]
			if (add_mw[i]>=0.1) printf "%.1f MW ", add_mw[i]
			if (add_vol[i]>=0.1) printf "%.1f m3 ", add_vol[i]
			if (add_mass[i]>=0.1) printf "%.1f tons ", add_mass[i]
			printf "\n"
			}

	printf "\nAccomodations:\n"
	printf "\tLife Support     : %s %.2f MW, %.1f m3, %d tons for %d m3\n",
		lstype, lsmw, lsvol, lsmass, lsload
	for (i =1; i<= add; i++)
		if (add_type[i]=="acco") {
			printf "\t%-16.16s : ", add_sect[i]
			acco_sect = add_sect[i]
			if (add_num[i]>1) printf "%d ", add_num[i]
			printf "%s ", add_name[i]
			if (add_mw[i]>=0.1) printf "%.1f MW ", add_mw[i]
			if (add_vol[i]>=0.1) printf "%.1f m3 ", add_vol[i]
			if (add_mass[i]>=0.1) printf "%.1f tons ", add_mass[i]
			printf "\n"
			}
	if (acco_sect != "Cargo")
		printf "\tCargo            : %d m3 %d dt\n", load*14, load
	else
		printf "\t                 : %d m3 %d dt\n", load*14, load
	}
END {
	if (system("tty >/dev/null")) { calc(); show() }
	}
